# -*- coding: utf-8 -*-
"A rectangle method with support for rounded corners"

__author__ = "Kamran Riaz Khan"
__email__ = "krkhan@inspirated.com"
__version__ = "1.1"
__copyright__ = "Copyright (c) 2009 Kamran Riaz Khan"
__license__ = "Python"
__status__ = "Production"

def rounded_rectangle(image, coords, radius, **options):
	"""Draw a rounded rectangle.

	Keyword arguments:
		image -- graphics.Image on which to build rounded rectangle
		coords -- 2-tuple of top-left and bottom-right coords
		radius -- rounding radius
		<option> -- drawing options"""
	# top-left corner
	a, b = coords[0]
	# bottom-right corner
	c, d = coords[1]

	coordseq = [
		(a, b + radius),
		(c, d - radius),
		(a + radius, b),
		(c - radius, d)
		]
	image.rectangle(coordseq, **options)

	coordseq = [
		(a, b),
		(a + 2 * radius, b + 2 * radius),
		(c - 2 * radius, b),
		(c, b + 2 * radius),
		(a, d - 2 * radius),
		(a + 2 * radius, d),
		(c - 2 * radius, d - 2 * radius),
		(c, d)
		]
	image.ellipse(coordseq, **options)

def rounded_rectangle_text(image, coords, radius, text, **options):
	"""Draw a rounded rectangle with text in its center.

	Keyword arguments:
		image -- graphics.Image on which to build rounded rectangle
		coords -- 2-tuple of top-left and bottom-right coords
		radius -- rounding radius
		text -- unicode text
		<option> -- drawing options"""
	# top-left corner
	a, b = coords[0]
	# bottom-right corner
	c, d = coords[1]
	# center coords
	cx, cy = (a + c) / 2.0, (b + d) / 2.0
	info = image.measure_text(text,
		maxwidth = int((c - radius) - (a + radius)))
	width, height, chars = info[0][2], abs(info[0][1]), info[2]

	rounded_rectangle(image, coords, radius, **options)
	image.text((cx - (width / 2.0), cy + (height / 2.0)), text[:chars])

def exit_key_handler():
	"Release app_lock."
	app_lock.signal()

if __name__ == "__main__":
	import appuifw, e32, graphics
	import scrolledcanvas

	image = graphics.Image.new((320, 240))
	canvas = scrolledcanvas.ScrolledCanvas(image)

	appuifw.app.title = u'Rounded Rectangle'
	appuifw.app.body = canvas
	appuifw.app.exit_key_handler = exit_key_handler

	rounded_rectangle_text(image, ((10, 10), (90, 30)),
		text = u"Hello", radius = 2, fill = 0xff0000)
	rounded_rectangle_text(image, ((10, 50), (90, 70)),
		text = u"World", radius = 2, fill = 0x00ff00)
	rounded_rectangle_text(image, ((10, 90), (90, 110)),
		text = u"I just can't shut up, can I?",
		radius = 2, fill = 0x0000ff)

	app_lock = e32.Ao_lock()
	app_lock.wait()

